#ifndef    _URL_STREAMS_H
#define    _URL_STREAMS_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef    WIN32
#include <windows.h>
#include <winsock.h>
#include <io.h>
#else  /* WIN32 */
#include <unistd.h>
#endif /* WIN32 */

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#define  STRCHR strchr
#define  MEMSET(x,y,z)  memset(x,y,z)
#define  MEMCPY(x,y,z)  memcpy(x,y,z)

extern void setSCError( char *);
extern char * getSCError();

#ifdef    S_SUNOS4
#define OLDFREE 1
#endif /* S_SUNOS4 */

#ifdef    OLDFREE
#define FREE(x)        free((char*)x)
#define REALLOC(x,y)   realloc((char*)x,y)
#else  /* OLDFREE */
#define FREE(x)        free((void*)x)
#define REALLOC(x,y)   realloc((void*)x,y)
#endif /* OLDFREE */

class InputStream
{
	public:
		InputStream() {}

		virtual int available() = 0;
		virtual int read() = 0;
		virtual int read( char * buf, int len) = 0;
		virtual int skip( int len) = 0;

		virtual void close() = 0;
};

class BufferedInputStream: public InputStream
{
	private:
		InputStream * inputstream;
		void        * data;
		int           bufsize;

		void doubleBuffer();

	public:
		char        * dptr;
		int           buflen;
		int           fillBuffer();

		BufferedInputStream( InputStream * i, int bs = 5021)
		{
			inputstream = i;
			bufsize     = bs;
			buflen      = 0;
			data        = (void*)0;
		}

		~BufferedInputStream()
		{
			if (data)
				(void) FREE(data);
		}

		int available() { return inputstream->available(); }
		int read() { // not used here
			char buf[1];
			return read( buf, 1);
		}

		char * currentPosition() { return dptr; }
		int     indexOf( char c);

		int read( char * buf, int len);
		int skip( int len);

		void close() {}
		void * readAll( int * sizep); // extra method
};

class StdInputStream: public InputStream
{
	protected:
		int id;
		int ateof;
		int dontblock;

	public:
		StdInputStream( int i=-1) { dontblock=0; id = i; ateof=0; }

		int isOk() { return (id != -1); }

		int read( char * buf, int len);
		int read() { // not used here
			char buf[1];
			return read( buf, 1);
		}

		int available() { return !ateof; }
		void setBlocking( int i)
		{
			dontblock = !i;
		}

		int skip( int i) { return 0; }

		void close() {}
};

class CachedInputStream: public InputStream
{
	private:
		InputStream * istream;
		char        * cache;
		char        * cacheptr;
		int           cachelen;

	public:
		CachedInputStream( InputStream * i) { istream = i; cachelen = 0;}
		void setCache( char * buf, int len) { cache = buf; cachelen = len;
						      cacheptr = cache;
						    }

		int available() {
			if (cachelen)
				return 1;
			else
				return istream->available();
		}

		int read() { // not used here
			char buf[1];
			return read( buf, 1);
		}
		int read( char * buf, int len);
		int skip( int len);

		void close() { istream->close(); }

};

class FileInputStream: public StdInputStream
{
	public:
		FileInputStream( char * f)
			:StdInputStream()
		{
			id = ::open( f, O_RDONLY, 0);
		}

		static FileInputStream * getFileInputStream( char * f)
		{
			FileInputStream * fs = new FileInputStream( f);
			if (!fs->isOk()) {
				delete fs;
				return (FileInputStream*)0;
			} else
				return fs;
		}

		void close() {
			::close( id);
		}
};

class DataInputStream: public BufferedInputStream
{
	public:
		DataInputStream( InputStream * i)
			:BufferedInputStream(i)
		{
		}

		char * readLine();
};

class InputData
{
	private:
		void * data;
		int    size;
		int    loaded;

		BufferedInputStream * istream;

		void loadData() { size = 0; data = istream->readAll( &size); }
		int checkData() {
			if (loaded)
				return 0;
			else {
				loadData();
				loaded = 1;
				return 0;
			}
		}

	public:
		InputData( InputStream * i)
		{
			istream = new BufferedInputStream(i);
			loaded  = 0;
		}

		~InputData()
		{
			if (data)
				FREE(data);
			if (istream)
				delete istream;
		}


		void * getData()
		{
			checkData();
			return data;
		}

		int getSize()
		{
			checkData();
			return size;
		}
};

class OutputStream
{
	public:
		virtual int write( char * buf, int len) = 0;

		virtual void flush() = 0;
		virtual void close() = 0;
};

class BufferedOutputStream: public OutputStream
{
	private:
		OutputStream * outputstream;
		char * data;
		char * dptr;
		int bufsize;
		int buflen;

	public:
		BufferedOutputStream( OutputStream * o, int size=5012)
		{
			outputstream = o;
			bufsize      = size;
		}

		int  write( char * buf, int len);
		void flush();
		void close() {
			flush();
			outputstream->close();
		}
};

class StdOutputStream: public OutputStream
{
	protected:
		int id;
		int length;

	public:
		StdOutputStream( int i=-1) { id = i; length = 0; }
		int isOk() { return (id != -1); }

		int write( char * buf, int len)
		{
			int n = ::write( id, buf, len);
			if (n != -1)
				length += n;

			return n;

		}

		void flush() {}
		void close() { (void) ::close( id); }
};

class FileOutputStream: public StdOutputStream
{
	protected:
		char * filename;

	public:
		FileOutputStream(char * f, int a=0, int rw=0)
		{
			int access  = (rw ? O_RDWR : O_WRONLY);
			int mode    = (a ? O_APPEND : O_TRUNC);
			id = open( f, access|O_CREAT|mode, 0744);
			filename = strdup( f);
		}

		static FileOutputStream * getFileOutputStream( char * f, int a=0)
		{
			FileOutputStream * fos = new FileOutputStream( f, a);
			if (!fos->isOk()) {
				delete fos;
				return (FileOutputStream*)0;
			} else
				return fos;
		}

		~FileOutputStream()
		{
			(void) FREE(filename);
		}


};

class TempFileOutputStream: public FileOutputStream
{
	public:
		TempFileOutputStream()
			:FileOutputStream( ::tmpnam((char*)0), 0, 1)
		{
			(void) ::unlink( filename);
		}
		~TempFileOutputStream()
		{
			(void) ::close( id);
		}

		static TempFileOutputStream * getTempFileOutputStream()
		{
			TempFileOutputStream * ts = new TempFileOutputStream();
			if (!ts->isOk()) {
				delete ts;
				return (TempFileOutputStream*)0;
			}
			return ts;
		}

		int getContentLength()
		{
			struct stat sb;

			if (::fstat( id, &sb) == -1)
				return -1;
			else
				return sb.st_size;
		}

		void copy( OutputStream * o);

		void close() { flush();}
};

class PrintWriter
{
	private:
		OutputStream * os;
		char * convbuf;

	public:
		PrintWriter( OutputStream * o) {
			os = o;
			convbuf = new char[64];
		}
		~PrintWriter() {
			delete convbuf;
			close();
		}

		void print  (char * s)  {
			if (s)  {
				os->write( s, ::strlen(s));
			}
		}
		void println( char * s) {
			print( s);
			os->write((char *) "\n", 1);
		}
		void println( int i) {
			(void) sprintf( convbuf, "%d", i);
			println( convbuf);
		}

		void close() {
			if (os)
				os->close();
		}
};

#ifdef    WIN32
#define EINTR WSAEINTR
#define EBADF WSAEBADF

typedef SOCKET SocketHandle;

class SocketInputStream: public StdInputStream
{
	private:
		SocketHandle handle;

	public:
		SocketInputStream( SocketHandle i) {
			handle= i;
			setBlocking(1);
		}

		int isOk() { return ( handle != INVALID_SOCKET); }

		int read( char * buf, int len) {
			return ::recv( handle, buf, len, 0);
		}

		void close() {
			::close( handle);
		}
};

class SocketOutputStream: public StdOutputStream
{
	private:
		SocketHandle handle;

	public:
		SocketOutputStream( SocketHandle i)
		{
			handle = i;
		}
		int write( char * buf, int len) {
			return ::send( handle, buf, len, 0);
		}
		void close() {
			::close( id);
		}
};

#else  /* WIN32 */
typedef int SocketHandle;

class SocketInputStream: public StdInputStream
{
	public:
		SocketInputStream( SocketHandle i) {
			id = i;
			setBlocking(1);
		}

		void close() {
			::close( id);
		}
};

typedef StdOutputStream SocketOutputStream;

#endif /* WIN32 */

#endif /* _URL_STREAMS_H */
