#ifndef _TOOLKIT_H
#define _TOOLKIT_H

#include "ToolkitDefs.h"

#ifndef WIN32
#include <xdclass.h>
#include <Xm/Xm.h>
#include <Xm/ToggleB.h>
#endif

// abstract event interface

class GEvent
{
	private:
		int x;
		int y;

	public:
		GEvent() {}
		GEvent(int xp, int yp)
		{
			setXY( xp, yp);
		}
		virtual int isButtonEvent() {  return 0; }
		virtual int isMotionEvent() {  return 0; }
		virtual int isKeyEvent()    {  return 0; }

		void setXY( int xp, int yp) { x = xp; y = yp; };

		int getX() { return x; }
		int getY() { return y; }
};

class GOnOffEvent: public GEvent
{
	private:
		int      isanOnsetEvent;

	public:
		GOnOffEvent() {
			isanOnsetEvent = 0;
		}

		GOnOffEvent(int xp, int yp)
			:GEvent( xp, yp)
		{
			isanOnsetEvent = 0;
		}

		void setOnsetEvent( int yesno)
		{
			isanOnsetEvent = yesno;
		}

		int isOnsetEvent() {
			return (isOnsetEvent != 0);
		}
};

class GButtonEvent: public GOnOffEvent
{
	private:
		GButton  currentButton;

	public:
		GButtonEvent( int xp, int yp)
			:GOnOffEvent( xp, yp)
		{}

		virtual int isButtonEvent() {  return 1; }

		GButton getButton() {
			return currentButton;
		}

		void setButton( GButton gb) { currentButton = gb; }
};

class GKeyEvent: public GOnOffEvent
{
	public:
		GKeyEvent( int xp, int yp)
			:GOnOffEvent( xp, yp) //Should this call the default constructor why x/y?
		{}
		virtual int isKeyEvent()    {  return 1; }
};

class GMotionEvent: public GEvent
{
	public:
		GMotionEvent( int xp, int yp)
			:GEvent( xp, yp)
		{}
		virtual int isMotionEvent() {  return 1; }
};

// Event Consumer Interface

class EventConsumer
{
	public:
		virtual int  needsMoreEvents() = 0;
		virtual int  consumeEvent( GEvent * ge) = 0;
		virtual int  isPersistent() { return 0; }
};

class EventConsumerItem
{
	private:
		EventConsumer * thisitem;
		EventConsumerItem * nextitem;

	public:
		EventConsumerItem( EventConsumer * ec, EventConsumerItem * nexti)
		{
			thisitem = ec;
			nextitem = nexti;
		}

		~EventConsumerItem()
		{
			if (!thisitem->isPersistent())
				delete thisitem;
		}

		EventConsumerItem * nextItem()
		{
			return nextitem;
		}

		void setNextItem( EventConsumerItem * ei) { nextitem = ei; }

		int consumeEvent( GEvent * ge)
		{
			return thisitem->consumeEvent( ge);
		}

		int needsMoreEvents()
		{
			return thisitem->needsMoreEvents();
		}
};

class FontMetrics
{
	protected:
		int isLeftToRight;
		int fontAscent;
		int fontDescent;
		FontObj * font;

	public:
		FontMetrics( FontObj * f)
		{
			font = f;
			isLeftToRight = 1;
		}

		int getIsLeftToRight() { return isLeftToRight; }
		int getFontAscent()    { return fontAscent; }
		int getFontDescent()   { return fontDescent; }

		virtual int getStringWidth( char * s) = 0;
};

#include "Graphics.h"

class PolygonRegion
{
	public:

		PolygonRegion() {}

		virtual PolygonRegion* getIntersection( PolygonRegion* p) = 0;

		virtual void * getData() = 0;

};


class MFCToolkit;
class XToolkit;

class Toolkit
{
	private:

		static Toolkit * theToolkit ;

		EventConsumerItem * eventConsumers;

	public:
		Toolkit() {
			eventConsumers = (EventConsumerItem*)0;
		}

		static Toolkit      * getDefaultToolkit();

		virtual FontMetrics   * getFontMetrics( FontObj * f) = 0;
		virtual PolygonRegion * getPolygonRegion( GPoint * points, int size) = 0;

		virtual Color * loadColor( char * rgb) = 0;
		virtual Color * getColor( GColor color) = 0;
		virtual int  init( CWnd * top, char * private_cmap) = 0;
		virtual int  setIcon( char * iconname, char * iconlabel) = 0;
		virtual void setIconFromData( char * bits, int width, int height) = 0;
		virtual void initDialog( CWnd * d) = 0;
		virtual int getWidth( CWnd * w) = 0;
		virtual int getHeight( CWnd * w) = 0;
		virtual void * getDisplay() = 0;
		virtual Graphics * getGraphics( CWnd *) = 0;
		virtual Graphics * getGraphics( CWnd *, int width, int height, Boolean invDisplay = NULL) = 0;

		//Convenience RadioBox functions.

		virtual int getSelectionByIndex (CWnd *radioBox) = 0;
		virtual void setSelectionByIndex (CWnd *radioBox, int val) = 0;
		virtual int getRBChildren(CWnd *RadioBox) = 0;

		virtual FontObj * getDefaultFont() = 0;
		virtual void defineCursor( CWnd * w, GCursor cursor) = 0;
		virtual void undefineCursor( CWnd * w) = 0;

		virtual void addEventConsumer( EventConsumer * ec)
		{
			printf("adding event consumer...\n");
			EventConsumerItem * i = new EventConsumerItem( ec, eventConsumers);
			eventConsumers = i;
		}

		virtual int hasAvailableEventConsumers()
		{
			return eventConsumers != (EventConsumerItem*)0;
		}

		virtual int processEvent( GEvent * ge)
		{
			EventConsumerItem * eci     = eventConsumers;
			EventConsumerItem * lastone = (EventConsumerItem*)0;



			for (; eci; lastone = eci, eci = eci->nextItem())
			{
				printf( "call ce\n");
				if (!eci->consumeEvent( ge))
					continue;
				if (eci->needsMoreEvents())
					return 1;
				if (lastone)
					lastone->setNextItem(eci->nextItem());
				else
					eventConsumers = (EventConsumerItem*)0;
				delete eci;
				return 1;
			}

			return 0;
		}

		virtual GEvent * newEvent( void* ev) = 0;
		virtual void     addWorkProcedure( void * proc, void * data) = 0;
		virtual void     updateDisplay() = 0;
};






#ifdef WIN32
#include "MFCToolkit.h"
#else
#include "XToolkit.h"
#endif
#endif
